/* clean_merge.do
This file merges nespd and other data in preparation for part B
*/

// set_globals needs to be run first

*** 1. Load and merge clean worker-level datasets

use "...\nesashe7520microclean.dta", clear

// keep spells where we observe earnings
drop if earn_weekly == .

// merge with GDP data
merge m:1 year using "...\gdpannualclean.dta", nogen

// merge with CPI data and deflate key variables
merge m:1 year using "...\cpiclean.dta", nogen

// merge with lower earnings threshold used in part a
merge m:1 year using "...\earnings_threshold.dta", nogen

*** 2. Further cleaning of variables

// restrict to people with one job
gen totaljobs = numfulltimejobs + numparttimejobs
label var totaljobs "Total jobs"
keep if totaljobs == 1

// deflate earnings and value added by CPI
foreach var of varlist earn* {
	replace `var' = `var'*100/cpi
	gen log_`var' = log(`var')
}
// rename where needed
gen log_thrs = log(thrs)
rename log_thrs log_hours

// trim and winsorize log earnings variables and hours
foreach var of varlist log_earn_weekly log_earn_hourly log_earn_annual log_hours {
	di "`var'"
	//gen `var'_tr = `var'
	gen `var'_w = `var'
	foreach yr of numlist 1975(1)2020 {
		di `yr'
		_pctile `var' if year == `yr', p(1 99)
		//replace `var'_tr = . if `var'_t <= r(r1) & year == `yr'
		//replace `var'_tr = . if `var'_t >= r(r2) & year == `yr'
		replace `var'_w = r(r1) if `var'_w <= r(r1) & year == `yr'
		replace `var'_w = r(r2) if `var'_w >= r(r2) & year == `yr' & `var'_w != .
	}
}

// residualize on year*age*sex
gen log_earn_weekly_w_res = .
foreach yr of numlist 1975(1)2020 {
	reg log_earn_weekly_w i.age##i.male if year == `yr'
	predict log_earn_weekly_w_res`yr', res
	replace log_earn_weekly_w_res = log_earn_weekly_w_res`yr' if year == `yr'
	drop log_earn_weekly_w_res`yr'
}

// Now calculate number of previous years available
sort piden year
cap drop log_earn_weekly_w_res_l*
cap drop year_l*
cap drop year_l*
cap drop numobs
foreach lag of numlist 1(1)3 {
	bys piden: gen log_earn_weekly_w_res_l`lag' = log_earn_weekly_w_res[_n-`lag']
	bys piden: gen year_l`lag' = year[_n -`lag']
	gen year_l`lag'_present = 0
	replace year_l`lag'_present = 1 if year_l`lag' >= year - 3 & year_l`lag' != .
}
gen numobs = year_l1_present + year_l2_present + year_l3_present

// now calculate permanent income
foreach lag of numlist 1(1)3 {
	replace  log_earn_weekly_w_res_l`lag' = . if  year_l`lag'_present == 0
	replace  log_earn_weekly_w_res_l`lag' = 0 if  log_earn_weekly_w_res_l`lag' == .
}
egen permearn = rowtotal(log_earn_weekly_w_res_l1 log_earn_weekly_w_res_l2 log_earn_weekly_w_res_l3)
replace permearn = permearn/numobs
replace permearn = . if numobs < 2
gen permearn_rank = .
foreach yr of numlist 1978(1)2020 {
	di `yr'
	xtile permearn_rank`yr' = permearn if year == `yr', nq(5)
	replace permearn_rank = permearn_rank`yr' if year == `yr'
	drop permearn_rank`yr'
}

drop numobs log_earn_weekly_w_res* year_l*

// make sure all labelling is clear and updated
label var age "Age"
label var yob "Year of birth"
label var sex "Sex (1 = m, 2 = f)"
label var gpox "Gpox"
label var he "Hourly earnings"
label var hexo "Hexo"
label var bhr "Bhr"
label var ovpay "Ovpay"
label var numfulltimejobs "Number of full time jobs"
label var numparttimejobs "Number of part time jobs"
label var public "Public sector"
label var new_job "New job"
label var male "Male"
label var pay_affected "Pay affected by absence"
label var pay_not_reported "Pay not reported"
label var skill_rank "Skill rank"
label var earn_weekly "Real weekly earnings"
label var earn_annual "Real annual earnings"
label var earn_hourly "Real hourly earnings"
label var union "Wages set by collective agreement?"
label var sic1968_div "Sic 1968 division"
label var sic1980_div "Sic 1980 division"
label var sic2003_div "Sic 2003 division"
label var entref "entref"
label var new_firm "New entref"
label var ft "Full time"
label var pt "Part time"
//label var last_year_observed "Last year appears in NESPD"
//label var first_year_observed "First year appears in NESPD"
//label var missingtype "Why are earnings missing for this observation?"
//label var _mergeardx "Merge with ARDx"
label var gdp "GDP (real)"
label var gdp_pch "percentage change in GDP (real)"
label var cpi "CPI"
label var log_earn_weekly "Real log weekly earnings"
//label var log_earn_weekly_tr "Real log weekly earnings (trimmed)"
label var log_earn_weekly_w "Real log weekly earnings (winsorized)"
label var log_earn_hourly "Real log hourly earnings"
//label var log_earn_hourly_tr "Real log hourly earnings (trimmed)"
label var log_earn_hourly_w "Real log hourly earnings (winsorized)"
label var log_earn_annual "Real log annual earnings"
//label var log_earn_annual_tr "Real log annual earnings (trimmed)"
label var log_earn_annual_w "Real log annual earnings (winsorized)"
//label var log_hours_tr "Log hours worked (trimmed)"
label var log_hours_w "Log hours worked (winsorized)"
label var log_hours "Log hours worked"
label var permearn "Average earnings in t-1,t-2,t-3 (residualized, numobs>=2)"
label var permearn_rank "Quintile rank of Average earnings in t-1,t-2,t-3 (residualized, numobs>=2)"
label var furlough "Worker furloughed"
label var paytoppedup "Furloughed worker pay topped up by employer"

// gen log gdp (not done before)
cap gen log_gdp = log(gdp)
label var log_gdp "Log real GDP"

// generate female dummy
gen female = 1 - male
label var female "Female"

// aggregate firm variables
gen firm_group=.
replace firm_group=1 if firm_size<100 & firm_size!=.
replace firm_group=2 if firm_size>=100 & firm_size<500
replace firm_group=3 if firm_size>=500 & firm_size<2000
replace firm_group=4 if firm_size>=2000 & firm_size!=.
label var firm_group "Firm size (grouped)"

// aggregate years
gen year_group=.
replace year_group=1 if year<1986
replace year_group=2 if year>=1986 & year<1996
replace year_group=3 if year>=1996 & year<2006
replace year_group=4 if year>=2006 
label var year_group "Years (grouped)"

// aggregate ages
gen age_group = .
replace age_group = 1 if age < 25
replace age_group = 2 if age >= 25 & age <35
replace age_group = 3 if age >= 35 & age <45
replace age_group = 4 if age >= 45 & age <55
replace age_group = 5 if age >= 55 
replace age_group = . if age == .

// recession variable
gen recession=0
replace recession=1 if loggdp1F<0
label var recession "Recession (GDP growth<0 between t and t+1)"

// generate within-worker log changes variables (use t to t+1)
// use raw earnings then winsorize and trim changes
gsort piden year
foreach var in log_earn_weekly log_earn_hourly log_hours log_earn_annual {
	by piden: gen lead1 = `var'[_n+1] if year == year[_n+1] - 1
	gen `var'1F = lead1 - `var'
	label var `var'1F "Change from t to t+1"
	drop lead1
	cap gen `var'1F_w = `var'1F
	foreach yr of numlist 1975(1)2019 {
		di `yr'
		_pctile `var'1F if year == `yr', p(1 99)
		//replace `var'_tr = . if `var'_t <= r(r1) & year == `yr'
		//replace `var'_tr = . if `var'_t >= r(r2) & year == `yr'
		replace `var'1F_w = r(r1) if `var'1F_w <= r(r1) & year == `yr'
		replace `var'1F_w = r(r2) if `var'1F_w >= r(r2) & year == `yr' & `var'1F_w != .
	}
//	winsor2 `var'1F, cuts(1 99) trim by(year)
//	winsor2 `var'1F, cuts(1 99) by(year)
}

// match match FE
egen matchFE = group(entref piden)

// make skill groups
gen skill_group = .
replace skill_group = 1 if skill_rank == 1 | skill_rank == 2
replace skill_group = 2 if skill_rank == 3 | skill_rank == 4
replace skill_group = 3 if skill_rank == 5 | skill_rank == 6
label var skill_group "Skill"
label define skill_group 1 "Low" 2 "Mid" 3 "High"
label values skill_group skill_group

// save worker version
save ".../partb_dat_clean_worker", replace

*** 3. Merge in firm data
// note can load from here to avoid running all above if only change firm data

use ".../partb_dat_clean_worker", clear
cap drop _merge
merge m:1 entref year using ".../firmdat_clean"
drop if _merge == 2 // drop unmatched firms
drop _merge

*** 4. Save cleaned data
save ".../partb_dat_clean", replace
