/* clean_firm.do
This file cleans all firm-level data
*/

// set_globals needs to be run first

// for testing

*** 1. BSD

foreach yr of numlist 2002(1)2018 {
	di "`yr'"
	use "...\6697_BSD_10th_Ed_Mar19\stata\stata11\eu_bsd_`yr'_restricted.dta", clear
	if `yr' >= 2017 {
		cap destring live_ru, force replace
		gen active = live_ru > 0
	}
	keep if active == 1
	cap destring employees, replace force
	cap destring turnover, replace force
	drop if employees == . | turnover == .
	cap tostring entref, replace force
	gsort entref -employees -turnover
	duplicates drop entref, force // very small number have duplicates, keep one with most employees
	keep entref employees turnover
	gen year = `yr'
	rename employees employees_bsd 
	rename turnover turnover_bsd
	save "...\bsd_clean`yr'.dta", replace
}

*** 2. FAME

*** 2.1 Main cleaning

use ".../FAME_IDBR_matched_Blundell_128380", clear

// keep only those which had a crn which matched to an entref
keep if _merge == 3

// make entrefs strings
tostring entref*, replace

// how many firms have different entrefs for different years of lookup?
gen entref_change = 0
foreach i of numlist 14(1)19 {
	foreach j of numlist 14(1)19 {
			replace entref_change = 1 if entref`i' != entref`j' & entref`i' != "." & entref`j' != "."
	}
}
tab entref_change

// how many matched firms are missing an entref for 14-19?
gen entref_miss = entref14 == "." | entref15 == "." | entref16 == "." | ///
entref17 == "." | entref19 == "."
tab entref_miss

// fill in from next if missing
foreach i of numlist 14(1)18 {
	foreach j of numlist 15(1)19 {
		if (`j' > `i' ){ 
			replace entref`i' = entref`j' if entref`i' == "." & entref`j' != "."
		}
	}
}

// fill in from previous if still missing
foreach i of numlist 19(-1)14 {
	foreach j of numlist 18(-1)14 {
		if `j'<`i' {
			replace entref`i' = entref`j' if entref`i' == "." & entref`j' != "."
		}
	}
}

// backfill 2013 to 2002 using oldest entref available
gen entref_backfill = entref14

foreach i of numlist 15(1)19 {
		replace  entref_backfill = entref`i' if entref_backfill == ""
}

foreach i in 00 01 02 03 04 05 06 07 08 09 10 11 12 13 {
		gen entref`i' = entref_backfill
}
drop *backfill *change *miss

// make long
gen tmpid = _n
reshape long operatingprofit20 pensioncosts20 socialsecuritycosts20 workingcapital20 wagessalaries20 otherstaffcosts20 numberofemployees20 turnover20 entref, i(tmpid) j(year) string
destring year, force replace
replace year = year + 2000
rename *20 *
drop if year == 2019
drop if year < 2002

// clean out missings
drop if numberofemp == . & turn == . & wages == . & op == . & pens ==. & soc == . & otherstaff ==. & workingcapital == .
drop if entref == ""
drop if entref == "."

// count minor staff costs as zero if missing
foreach var of varlist pension social otherstaff {
	replace `var' = 0 if `var' == .
}

// save year by year
foreach yr of numlist 2002(1)2018 {
	preserve
	keep if year == `yr'
	save "...\fame_`yr'.dta", replace
	restore
}


// merge with BSD and drop all incorrectly matched crn-entrefs
foreach yr of numlist 2002(1)2018 {
	use "...\fame_`yr'.dta", clear
	cap drop _merge
	merge m:1 entref using "...\bsd_clean`yr'.dta"
	tab _merge
	keep if _merge == 3
	drop _merge
	// drop those which are 50% off their BSD figure
	gen employment_fame_bsd = numberofemp/employees_bsd
	gen turnover_fame_bsd = turnover/turnover_bsd
	drop if employment_fame_bsd < 0.5 | employment_fame_bsd > 1.5
	drop if turnover_fame_bsd < 0.5 | turnover_fame_bsd > 1.5

	// if still duplicates, take the one most similar on employment, then turnover
	gen emp_fac = (1 - employment_fame_bsd)^2
	gen turnover_fac = (1 - turnover_fame_bsd)^2
	sort entref emp_fac turnover_fac numberofemp employees_bsd turnover operatingprofit wages pension social otherstaff numberofemp sic2007_1 sic2007_2 sic2007_3
	by entref: gen ind = [_n]
	drop if ind > 1
	
	// drop variables
	drop ind
	save "...\fame_clean_`yr'.dta", replace
}

// append across years
use "...\fame_clean_2002.dta", replace
foreach yr of numlist 2003(1)2018 {
	append using "...\fame_clean_`yr'.dta"
}


// generate value added variables
gen vapw_fame = (operatingprofit + wages + pension + social + otherstaff)/numberofemp
gen log_vapw_fame = log(vapw_fame)
gen va_fame = operatingprofit + wages + pension + social + otherstaff
gen log_va_fame = log(va_fame)
label var vapw_fame "Value added pw (FAME)"
label var log_vapw_fame "log Value added pw (FAME)"
label var va_fame "Value added (FAME)"
label var log_va_fame "log Value added (FAME)"

// drop if missing value added per worker
drop if vapw_fame == .

// generate winsorized version
winsor2 log_vapw_fame, cuts(1 99) by(year)

// industry variables
gen sic = sic2007_1
replace sic = sic2007_2 if sic == .
replace sic = sic2007_3 if sic == .
rename sic sic_fame
label var sic_fame "SIC code (FAME)"

// rename and drop other variables
rename turnover turnover_fame
rename numberofemployees numberofemployees_fame
rename workingcapital workingcapital_fame
keep *fame *_w entref year emp_fac *_bsd

destring entref, replace force

*** 2.2 Generate instrument

// generate indicator for whether in instrument calculation
gen in_inst = 0
replace in_inst = 1 if va_fame != . & numberofemp != .

tostring sic, gen(sic_string)
replace sic_string = "0" + sic_string if length(sic_string) == 4
gen sic2dig_fame = substr(sic_string,1,2)
gen sic3dig_fame = substr(sic_string,1,3)
gen sic4dig_fame = substr(sic_string,1,4)
foreach i of numlist 2(1)4 {
	// version which uses sums of VA and employment
	egen sumva_sic`i'dig_fame = sum(va_fame) if in_inst == 1, by(year sic`i'dig_fame)
	egen sumemp_sic`i'dig_fame = sum(numberofemp) if in_inst == 1, by(year sic`i'dig_fame)
	gen fame_instrument_`i'dig = log((sumva_sic`i'dig_fame - va_fame)/(sumemp_sic`i'dig_fame - numberofemp))
	winsor2 fame_instrument_`i'dig, cuts(1 99) by(year)
	
	// version which takes weighted mean of logs
	egen sum_employment = sum(numberofemp) if in_inst == 1, by(year sic`i'dig_fame)
	bys year sic`i'dig_fame: gen sum_leaveout_employment = sum_employment - numberofemp if in_inst == 1 
	gen weighted_log_vapw = numberofemp*log_vapw_fame/sum_leaveout_employment
	egen leaveoutsum = sum(weighted_log_vapw), by(year sic`i'dig_fame)
	gen fame_instrument_`i'dig_alt = leaveoutsum - weighted_log_vapw
	drop leaveoutsum sum_employment sum_leaveout_employment  weighted_log_vapw
	winsor2 fame_instrument_`i'dig_alt, cuts(1 99) by(year)	
}

*** 2.3 Save

save ".../fame_clean", replace

*** 3. ARDx

*** 3.1 Clean ARDx at reporting unit level

// load universe files
clear
foreach year of numlist 2002/2014 {
di `year'
use year entref ruref employment_IDBR using "...\7989_ARDx_3rd_Ed_Nov16\stata\stata11\ardx_ru_v3.2\universe_ru_`year'_restricted.dta", clear
// drop any ru's with zero employment
drop if employment_IDBR == 0
save ".../universe_ru_`year'", replace
}

// main ARDx data
clear
foreach year of numlist 2002/2014 {
di `year'
append using "...\7989_ARDx_3rd_Ed_Nov16\stata\stata11\ardx_v3.2\ardx`year'_restricted.dta", keep(year entref ruref employment_IDBR employment_ya_total employees_pit_total turnover sic07 wq615 wq446 wq611 wq612 wq613 wq630 wq632 wq252)
}

// rename variables
rename wq446 wagessalaries
rename wq611 gvamarketprices
rename wq612 gvafactorcost
rename wq613 gvabasicprices
rename wq615 grossoperatingsurplus
rename wq252 totalemploymentcosts

// note that can vary this one depending on what employment var want to use
rename employment_IDBR employment

replace sic07 = "0" + sic07 if length(sic07) == 4
gen sic2dig_ardx = substr(sic07,1,2)
gen sic3dig_ardx = substr(sic07,1,3)
gen sic4dig_ardx = substr(sic07,1,4)
destring sic*ardx, force replace

// drop if no employment
drop if employment == 0

save ".../ardx_ruref_clean.dta", replace

*** 3.2. Diagnostics of rerporting unit level ardx
// Generate reference table noting which entrefs correspond to their full set of RUs, according to the universe file
// here I check using the universe files, whether when a reporting unit is sampled, are all 
// other reporting units in the same enterprise unit sampled?

foreach year of numlist 2002/2014 {
	di `year'
	use ".../universe_ru_`year'", clear
	merge 1:1 ruref year using ".../ardx_ruref_clean.dta", force
	drop if year != `year'
	gen inARD = _merge == 3
	collapse inARD, by(entref year)
	drop if inARD == 0
	gen missingru = 0
	replace missingru = 1 if inARD > 0 & inARD < 1 // these entrefs are missing some reporting units
	tab missingru
	drop inARD
save ".../missingru`year'", replace
}
// there are quite a few entrefs with reporting units in idbr which don't appear in ARDx. Not clear why

*** 3.3. Clean ARDx at entref level

use ".../ardx_ruref_clean.dta", clear
// merge in indicator for whether it looks like there are missing RU's within an entref
foreach year of numlist 2002/2014 {
merge m:1 entref year using ".../missingru`year'", update
drop _merge
}

// drop those with missing entrefs
destring entref, replace force
gen missentref = entref == .
tab missentref
drop if missentref == 1
drop missentref

// generate indicator for cases where an entref has multiple ruref
bys entref year: gen tick = _n
egen numrurefs = max(tick), by(entref year)

// need indicators for whether an ruref is missing a variable
// missings
foreach var in employment turnover wagessalaries gvamarket ///
gvafactorcost gvabasicprices grossoperatingsurplus totalemploymentcosts {
	gen miss_`var' = `var' == .
}

// collapse by entref
collapse (sum) employment turnover wagessalaries gvamarket gvafactorcost gvabasicprices grossoperatingsurplus totalemploymentcosts  (mean) missingru numrurefs (max) sic*ardx miss_*, by(entref year)

foreach var in  turnover wagessalaries gvamarket ///
gvafactorcost gvabasicprices grossoperatingsurplus totalemploymentcosts {
	replace `var' = . if miss_`var' ==1
}
drop miss_*

// generate per-worker variables
foreach var in turnover wagessalaries gvamarketprices ///
gvafactorcost gvabasicprices grossoperatingsurplus totalemploymentcosts {
	gen `var'_pw = `var'/employment
}

// generate logs
foreach var in employment  gvamarketprices_pw ///
gvafactorcost_pw gvabasicprices_pw turnover wagessalaries gvamarketprices ///
gvafactorcost gvabasicprices grossoperatingsurplus totalemploymentcosts {
	gen log_`var' = log(`var')
}

// drop all enterprises with no employment
drop if employment == 0

// keep limited variables and rename
rename log_gvafactorcost_pw log_vapw_ardx
label var log_vapw_ardx "Log value added per worker (ARDx)"
rename log_gvafactorcost log_va_ardx
label var log_va_ardx "Log value added (ARDx)"
rename gvafactorcost_pw vapw_ardx
label var vapw_ardx "Value added per worker (ARDx)"
rename gvafactorcost va_ardx
label var va_ardx "Value added (ARDx)"
label var missingru "entref is missing ru in ARDx"

keep entref year log_vapw_ardx*  log_va_ardx* vapw_ardx* va_ardx* missingru employment sic*ardx

// generate trimmed and winsorized versions of VA variables
foreach var of varlist log_vapw_ardx*  log_va_ardx* vapw_ardx* va_ardx* {
	winsor2 `var', cuts(1 99) by(year)
}

*** 3.4. Generate ARDx instrument

gen in_inst = va_ardx != . & employment != . & sic2dig_ardx != .
foreach i of numlist 2(1)4 {
	// version which takes log of sums
	egen sumva_sic`i'dig_ardx = sum(va_ardx) if in_inst == 1, by(year sic`i'dig_ardx)
	egen sumemp_sic`i'dig_ardx = sum(employment) if in_inst == 1, by(year sic`i'dig_ardx)
	gen ardx_instrument_`i'dig = log((sumva_sic`i'dig_ardx - va_ardx)/(sumemp_sic`i'dig_ardx - employment))
	winsor2 ardx_instrument_`i'dig, cuts(1 99) by(year)
	
	// version which takes weighted mean of logs
	egen sum_employment = sum(employment) if in_inst == 1, by(year sic`i'dig_ardx)
	bys year sic`i'dig_ardx: gen sum_leaveout_employment = sum_employment - employment if in_inst == 1 
	gen weighted_log_vapw = employment*log_vapw_ardx/sum_leaveout_employment
	egen leaveoutsum = sum(weighted_log_vapw), by(year sic`i'dig_ardx)
	gen ardx_instrument_`i'dig_alt = leaveoutsum - weighted_log_vapw
	drop leaveoutsum sum_employment sum_leaveout_employment  weighted_log_vapw
	winsor2 ardx_instrument_`i'dig_alt, cuts(1 99) by(year)	
}
drop in_inst 

/* previous way of generating instrument (for reference):

drop if gvafactorcost==.
sort year sic07_3d
by year sic07_3d: egen gvafactorcost_sector = sum(gvafactorcost)
by year sic07_3d: egen employment_sector = sum(employment_IDBR)
gen instrument_3d = ((gvafactorcost_sector-gvafactorcost)/(employment_sector-employment_IDBR))
drop gvafactorcost_sector employment_sector
sort year sic07_4d
by year sic07_4d: egen gvafactorcost_sector = sum(gvafactorcost)
by year sic07_4d: egen employment_sector = sum(employment_IDBR)
gen instrument_4d = ((gvafactorcost_sector-gvafactorcost)/(employment_sector-employment_IDBR))
keep instrument_3d instrument_4d year entref
save "$analysisdir\input\ardx_instrument", replace
*/

*** 3.5 Generate lagged VA ARDx instrument

sort entref year 
by entref: gen log_vapw_ardx_w_lag = log_vapw_ardx_w[_n-1] if year == year[_n-1] + 1
label var log_vapw_ardx_w_lag "Lagged VAPW (ARDx)"

*** 3.6. Save cleaned dataset

rename employment employment_ardx
save ".../ardx_entref_clean.dta", replace

*** 4. Combine FAME and ARDX

use ".../ardx_entref_clean.dta", clear
merge 1:1 entref year using  ".../fame_clean.dta"

// label vars

label var numberofemployees_fame "Number of employees in FAME"
label var turnover_fame "Turnover in FAME"
label var workingcapital_fame "Working capital in FAME"
label var log_vapw_ardx_w "Log value added per worker (ARDx, winsorized)"
label var log_vapw_fame_w "Log value added per worker (FAME, winsorized)"
label var log_vapw_ardx_w "Value added per worker (ARDx, winsorized)"
label var log_vapw_fame_w "Value added per worker (FAME, winsorized)"

// ad hoc vreate winsorized VA in fame
winsor2 vapw_fame, by(year)

// rename fame employment var
rename numberofemp employment_fame

// save cleaned firm dataset
drop _merge
save ".../firmdat_clean", replace